@@ -42,8 +42,8 @@ static GSList *override_search_path = NULL;
 
 struct _GIRepositoryPrivate
 {
-  GHashTable *typelibs; /* (string) namespace -> GTypelib */
-  GHashTable *lazy_typelibs; /* (string) namespace-version -> GTypelib */
+  GHashTable *typelibs; /* (string) namespace -> GITypelib */
+  GHashTable *lazy_typelibs; /* (string) namespace-version -> GITypelib */
   GHashTable *info_by_gtype; /* GType -> GIBaseInfo */
 };
 
@@ -168,19 +168,21 @@ g_irepository_get_search_path (void)
   return search_path;
 }
 
-static
-GSList *
+static GSList *
 build_search_path_with_overrides (void)
 {
-	GSList *result;
-	if (override_search_path != NULL)
-	  {
-		result = g_slist_copy (override_search_path);
-		g_slist_last (result)->next = g_slist_copy (search_path);
-	  }
-	else
-	  result = g_slist_copy (search_path);
-	return result;
+  GSList *result;
+
+  init_globals ();
+
+  if (override_search_path != NULL)
+    {
+      result = g_slist_copy (override_search_path);
+      g_slist_last (result)->next = g_slist_copy (search_path);
+    }
+  else
+    result = g_slist_copy (search_path);
+  return result;
 }
 
 static char *
@@ -193,7 +195,7 @@ build_typelib_key (const char *name, const char *source)
 }
 
 static char **
-get_typelib_dependencies (GTypelib *typelib)
+get_typelib_dependencies (GITypelib *typelib)
 {
   Header *header;
   const char *dependencies_glob;
@@ -218,8 +220,8 @@ get_repository (GIRepository *repository)
     return default_repository;
 }
 
-static GTypelib *
-check_version_conflict (GTypelib *typelib,
+static GITypelib *
+check_version_conflict (GITypelib *typelib,
 			const gchar *namespace,
 			const gchar *expected_version,
 			char       **version_conflict)
@@ -249,7 +251,7 @@ check_version_conflict (GTypelib *typelib,
   return typelib;
 }
 
-static GTypelib *
+static GITypelib *
 get_registered_status (GIRepository *repository,
 		       const char   *namespace,
 		       const char   *version,
@@ -257,7 +259,7 @@ get_registered_status (GIRepository *repository,
 		       gboolean     *lazy_status,
 		       char        **version_conflict)
 {
-  GTypelib *typelib;
+  GITypelib *typelib;
   repository = get_repository (repository);
   if (lazy_status)
     *lazy_status = FALSE;
@@ -274,7 +276,7 @@ get_registered_status (GIRepository *repository,
   return check_version_conflict (typelib, namespace, version, version_conflict);
 }
 
-static GTypelib *
+static GITypelib *
 get_registered (GIRepository *repository,
 		const char   *namespace,
 		const char   *version)
@@ -284,7 +286,7 @@ get_registered (GIRepository *repository,
 
 static gboolean
 load_dependencies_recurse (GIRepository *repository,
-			   GTypelib     *typelib,
+			   GITypelib     *typelib,
 			   GError      **error)
 {
   char **dependencies;
@@ -324,7 +326,7 @@ static const char *
 register_internal (GIRepository *repository,
 		   const char   *source,
 		   gboolean      lazy,
-		   GTypelib     *typelib,
+		   GITypelib     *typelib,
 		   GError      **error)
 {
   Header *header;
@@ -382,13 +384,13 @@ register_internal (GIRepository *repository,
  * Note: The namespace must have already been loaded using a function
  * such as #g_irepository_require before calling this function.
  *
- * Returns: Zero-terminated string array of versioned dependencies
+ * Returns: (transfer full): Zero-terminated string array of versioned dependencies
  */
 char **
 g_irepository_get_dependencies (GIRepository *repository,
 				const char *namespace)
 {
-  GTypelib *typelib;
+  GITypelib *typelib;
 
   g_return_val_if_fail (namespace != NULL, NULL);
 
@@ -402,7 +404,7 @@ g_irepository_get_dependencies (GIRepository *repository,
 
 const char *
 g_irepository_load_typelib (GIRepository *repository,
-			    GTypelib     *typelib,
+			    GITypelib     *typelib,
 			    GIRepositoryLoadFlags flags,
 			    GError      **error)
 {
@@ -497,7 +499,7 @@ gint
 g_irepository_get_n_infos (GIRepository *repository,
 			   const gchar  *namespace)
 {
-  GTypelib *typelib;
+  GITypelib *typelib;
   gint n_interfaces = 0;
 
   g_return_val_if_fail (namespace != NULL, -1);
@@ -529,7 +531,7 @@ find_interface (gpointer key,
 		gpointer data)
 {
   gint i;
-  GTypelib *typelib = (GTypelib *)value;
+  GITypelib *typelib = (GITypelib *)value;
   Header *header = (Header *) typelib->data;
   IfaceData *iface_data = (IfaceData *)data;
   gint index;
@@ -625,7 +627,7 @@ find_interface (gpointer key,
  * given namespace @namespace_.  The namespace must have
  * already been loaded before calling this function.
  *
- * Returns: #GIBaseInfo containing metadata
+ * Returns: (transfer full): #GIBaseInfo containing metadata
  */
 GIBaseInfo *
 g_irepository_get_info (GIRepository *repository,
@@ -633,7 +635,7 @@ g_irepository_get_info (GIRepository *repository,
 			gint          index)
 {
   IfaceData data;
-  GTypelib *typelib;
+  GITypelib *typelib;
 
   g_return_val_if_fail (namespace != NULL, NULL);
 
@@ -666,7 +668,7 @@ g_irepository_get_info (GIRepository *repository,
  * arbitrary GType - thus, this function will operate most reliably
  * when you know the GType to originate from be from a loaded namespace.
  *
- * Returns: #GIBaseInfo representing metadata about @type, or %NULL
+ * Returns: (transfer full): #GIBaseInfo representing metadata about @type, or %NULL
  */
 GIBaseInfo *
 g_irepository_find_by_gtype (GIRepository *repository,
@@ -719,7 +721,7 @@ g_irepository_find_by_gtype (GIRepository *repository,
  * #g_irepository_require once to load the namespace, or otherwise
  * ensure the namespace has already been loaded.
  *
- * Returns: #GIBaseInfo representing metadata about @name, or %NULL
+ * Returns: (transfer full): #GIBaseInfo representing metadata about @name, or %NULL
  */
 GIBaseInfo *
 g_irepository_find_by_name (GIRepository *repository,
@@ -727,7 +729,7 @@ g_irepository_find_by_name (GIRepository *repository,
 			    const gchar  *name)
 {
   IfaceData data;
-  GTypelib *typelib;
+  GITypelib *typelib;
 
   g_return_val_if_fail (namespace != NULL, NULL);
 
@@ -759,7 +761,7 @@ collect_namespaces (gpointer key,
 }
 
 /**
- * g_irepository_get_namespaces:
+ * g_irepository_get_loaded_namespaces:
  * @repository: A #GIRepository, may be %NULL for the default
  *
  * Return the list of currently loaded namespaces.
@@ -804,7 +806,7 @@ const gchar *
 g_irepository_get_version (GIRepository *repository,
 			   const gchar  *namespace)
 {
-  GTypelib *typelib;
+  GITypelib *typelib;
   Header *header;
 
   g_return_val_if_fail (namespace != NULL, NULL);
@@ -838,7 +840,7 @@ const gchar *
 g_irepository_get_shared_library (GIRepository *repository,
 				  const gchar  *namespace)
 {
-  GTypelib *typelib;
+  GITypelib *typelib;
   Header *header;
 
   g_return_val_if_fail (namespace != NULL, NULL);
@@ -874,7 +876,7 @@ const gchar *
 g_irepository_get_c_prefix (GIRepository *repository,
                             const gchar  *namespace_)
 {
-  GTypelib *typelib;
+  GITypelib *typelib;
   Header *header;
 
   g_return_val_if_fail (namespace_ != NULL, NULL);
@@ -1166,7 +1168,8 @@ find_namespace_latest (const gchar  *namespace,
  * @repository: (allow-none): the repository
  * @namespace_: GI namespace, e.g. "Gtk"
  *
- * Obtain a list of versions for @namespace_ in this @repository.
+ * Obtain an unordered list of versions (either currently loaded or
+ * available) for @namespace_ in this @repository.
  *
  * Returns: (element-type utf8) (transfer full): the array of versions.
  */
@@ -1177,6 +1180,7 @@ g_irepository_enumerate_versions (GIRepository *repository,
   GList *ret = NULL;
   GSList *search_path;
   GSList *candidates, *link;
+  const gchar *loaded_version;
 
   search_path = build_search_path_with_overrides ();
   candidates = enumerate_namespace_versions (namespace_, search_path);
@@ -1185,14 +1189,26 @@ g_irepository_enumerate_versions (GIRepository *repository,
   for (link = candidates; link; link = link->next)
     {
       struct NamespaceVersionCandidadate *candidate = link->data;
-      ret = g_list_append (ret, g_strdup (candidate->version));
+      ret = g_list_prepend (ret, g_strdup (candidate->version));
       free_candidate (candidate);
     }
   g_slist_free (candidates);
+
+  /* The currently loaded version of a namespace is also part of the
+   * available versions, as it could have been loaded using
+   * require_private().
+   */
+  if (g_irepository_is_registered (repository, namespace_, NULL))
+    {
+      loaded_version = g_irepository_get_version (repository, namespace_);
+      if (loaded_version && !g_list_find_custom (ret, loaded_version, g_str_equal))
+        ret = g_list_prepend (ret, g_strdup (loaded_version));
+    }
+
   return ret;
 }
 
-static GTypelib *
+static GITypelib *
 require_internal (GIRepository  *repository,
 		  const gchar   *namespace,
 		  const gchar   *version,
@@ -1201,9 +1217,9 @@ require_internal (GIRepository  *repository,
 		  GError       **error)
 {
   GMappedFile *mfile;
-  GTypelib *ret = NULL;
+  GITypelib *ret = NULL;
   Header *header;
-  GTypelib *typelib = NULL;
+  GITypelib *typelib = NULL;
   const gchar *typelib_namespace, *typelib_version;
   gboolean allow_lazy = (flags & G_IREPOSITORY_LOAD_FLAG_LAZY) > 0;
   gboolean is_lazy;
@@ -1321,9 +1337,9 @@ require_internal (GIRepository  *repository,
  * version @version of namespace may be specified.  If @version is
  * not specified, the latest will be used.
  *
- * Returns: a pointer to the #GTypelib if successful, %NULL otherwise
+ * Returns: (transfer full): a pointer to the #GITypelib if successful, %NULL otherwise
  */
-GTypelib *
+GITypelib *
 g_irepository_require (GIRepository  *repository,
 		       const gchar   *namespace,
 		       const gchar   *version,
@@ -1331,7 +1347,7 @@ g_irepository_require (GIRepository  *repository,
 		       GError       **error)
 {
   GSList *search_path;
-  GTypelib *typelib;
+  GITypelib *typelib;
 
   search_path = build_search_path_with_overrides ();
   typelib = require_internal (repository, namespace, version, flags,
@@ -1356,9 +1372,9 @@ g_irepository_require (GIRepository  *repository,
  * version @version of namespace should be specified.  If @version is
  * not specified, the latest will be used.
  *
- * Returns: a pointer to the #GTypelib if successful, %NULL otherwise
+ * Returns: (transfer full): a pointer to the #GITypelib if successful, %NULL otherwise
  */
-GTypelib *
+GITypelib *
 g_irepository_require_private (GIRepository  *repository,
 			       const gchar   *typelib_dir,
 			       const gchar   *namespace,
@@ -1378,8 +1394,15 @@ g_irepository_introspect_cb (const char *option_name,
 			     gpointer data,
 			     GError **error)
 {
-  gboolean ret = g_irepository_dump (value, error);
-  exit (ret ? 0 : 1);
+  GError *tmp_error = NULL;
+  gboolean ret = g_irepository_dump (value, &tmp_error);
+  if (!ret)
+    {
+      g_error ("Failed to extract GType data: %s",
+	       tmp_error->message);
+      exit (1);
+    }
+  exit (0);
 }
 
 static const GOptionEntry introspection_args[] = {
@@ -1389,6 +1412,15 @@ static const GOptionEntry introspection_args[] = {
   { NULL }
 };
 
+/**
+ * g_irepository_get_option_group: (skip)
+ *
+ * Obtain the option group for girepository, it's used
+ * by the dumper and for programs that wants to provide
+ * introspection information
+ *
+ * Returns: (transfer full): the option group
+ */
 GOptionGroup *
 g_irepository_get_option_group (void)
 {
@@ -1424,27 +1456,27 @@ g_type_tag_to_string (GITypeTag type)
     case GI_TYPE_TAG_VOID:
       return "void";
     case GI_TYPE_TAG_BOOLEAN:
-      return "boolean";
+      return "gboolean";
     case GI_TYPE_TAG_INT8:
-      return "int8";
+      return "gint8";
     case GI_TYPE_TAG_UINT8:
-      return "uint8";
+      return "guint8";
     case GI_TYPE_TAG_INT16:
-      return "int16";
+      return "gint16";
     case GI_TYPE_TAG_UINT16:
-      return "uint16";
+      return "guint16";
     case GI_TYPE_TAG_INT32:
-      return "int32";
+      return "gint32";
     case GI_TYPE_TAG_UINT32:
-      return "uint32";
+      return "guint32";
     case GI_TYPE_TAG_INT64:
-      return "int64";
+      return "gint64";
     case GI_TYPE_TAG_UINT64:
-      return "uint64";
+      return "guint64";
     case GI_TYPE_TAG_FLOAT:
-      return "float";
+      return "gfloat";
     case GI_TYPE_TAG_DOUBLE:
-      return "double";
+      return "gdouble";
     case GI_TYPE_TAG_GTYPE:
       return "GType";
     case GI_TYPE_TAG_UTF8:
