@@ -20,4 +20,6 @@
 
 DATADIR = "${prefix}/share"
 DATADIR = DATADIR.replace(
-    "${prefix}", "/home/johan/.local")
+    "${prefix}", "/usr/local")
+GIR_DIR = "/usr/local/share/gir-1.0"
+GIR_SUFFIX = "gir-1.0"
